#include "opencv2/imgcodecs.hpp"
#include "opencv2/imgproc.hpp"
#include "opencv2/videoio.hpp"
#include <opencv2/highgui.hpp>
#include <opencv2/video.hpp>

#include <iostream>

using namespace cv;

Mat detectHoughLines(const Mat& src)
{
     Mat edgeImage, result;
     Canny(src, edgeImage, 50, 200, 3);
     cvtColor(edgeImage, result, CV_GRAY2BGR);

     std::vector<Vec4i> lines;
     HoughLinesP(edgeImage, lines, 1, CV_PI / 180, 50, 20, 10);
     for (size_t i = 0; i < lines.size(); i++) {
             Vec4i l = lines[i];
             line(result, Point(l[0], l[1]), Point(l[2], l[3]), 
                     Scalar(0, 0, 255), 2, CV_AA);
     }
     return result;
}

Mat detectHoughCircles(const Mat &src) {
    Mat result, gray;

    cvtColor(src, gray, CV_BGR2GRAY);

    // opcionlis zajszrs
    GaussianBlur( gray, gray, Size(5, 5), 2, 2 );

    std::vector<Vec3f> circles;
    HoughCircles(gray, circles, CV_HOUGH_GRADIENT,
                 2, gray.rows/8, 100, 20, 0, 50 );

    src.copyTo(result);
    for( size_t i = 0; i < circles.size(); i++ )
    {
         Point center(cvRound(circles[i][0]), cvRound(circles[i][1]));
         int radius = cvRound(circles[i][2]);
         // kzppont
         circle( result, center, 3, Scalar(0,255,0), -1, 8, 0 );
         // krvonal
         circle( result, center, radius, Scalar(0,0,255), 3, 8, 0 );
    }
    return result;
}

int main(int argc, char* argv[])
{
	VideoCapture capture;
	if(argc < 2)
		capture.open(2); 
	else {
		capture.open(argv[1]);
		capture.set(CAP_PROP_FRAME_WIDTH, 640);
		capture.set(CAP_PROP_FRAME_HEIGHT, 480);
	}

	Mat overlay = imread("bme_cimer.jpg");

	namedWindow("PreviewWindow");
	Mat frame;
	Mat resized;
	Mat processed;
	std::vector<std::vector<Point> > contours;
	while(capture.read(frame)) {
		resize(frame, resized, Size(640,480));

		processed = detectHoughLines(resized);

		cv::imshow("PreviewWindow", processed);

		char key = static_cast<char>(waitKey(1));
		if (key == 27) { printf("exit"); break; }
	}
}
